# Cog Stuff
from discord.ext import commands
from discord.embeds import Embed
from discord.colour import Color

import asyncio
from datetime import datetime

import logging
logger = logging.getLogger(__name__)


class Remind(commands.Cog):
    """
    Set reminders
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True, case_insensitive=True, aliases=["remind", "remindme", "remind_me"])
    @commands.bot_has_permissions(attach_files=True, embed_links=True)
    async def reminder(self, ctx, time, *, reminder):
        print(time)
        print(reminder)
        embed = Embed(color=0x55a7f7, timestamp=datetime.utcnow())
        seconds = 0
        if reminder is None:
            # Error message
            embed.add_field(
                name='Warning', value='Please specify what do you want me to remind you about.')
        if time.lower().endswith("d"):
            seconds += int(time[:-1]) * 60 * 60 * 24
            counter = f"{seconds // 60 // 60 // 24} days"
        if time.lower().endswith("h"):
            seconds += int(time[:-1]) * 60 * 60
            counter = f"{seconds // 60 // 60} hours"
        elif time.lower().endswith("m"):
            seconds += int(time[:-1]) * 60
            counter = f"{seconds // 60} minutes"
        elif time.lower().endswith("s"):
            seconds += int(time[:-1])
            counter = f"{seconds} seconds"
        if seconds == 0:
            embed.add_field(name='Warning',
                            value='Please specify a proper duration, send `reminder_help` for more information.')
        elif seconds < 300:
            embed.add_field(name='Warning',
                            value='You have specified a too short duration!\nMinimum duration is 5 minutes.')
        elif seconds > 7776000:
            embed.add_field(
                name='Warning', value='You have specified a too long duration!\nMaximum duration is 90 days.')
        else:
            await ctx.send(f"Alright, I will remind you about {reminder} in {counter}.")
            await asyncio.sleep(seconds)
            await ctx.send(f"Hi, you asked me to remind you about {reminder} {counter} ago.")
            return
        await ctx.send(embed=embed)


def setup(bot):
    bot.add_cog(Remind(bot))



# Cog Stuff
from aadiscordbot.cogs.utils.decorators import message_in_channels
from discord.ext import commands
from discord.embeds import Embed
from discord.colour import Color
# AA Contexts
from django.utils import timezone
from django.conf import settings
from allianceauth.timerboard.models import Timer
from aadiscordbot.app_settings import timerboard_active

import datetime

import pendulum
import logging
logger = logging.getLogger(__name__)


class Timers(commands.Cog):
    """
    TimerBoard Stuffs!
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    @message_in_channels(settings.ADMIN_DISCORD_BOT_CHANNELS)
    async def timer(self, ctx):
        """
        Gets the Next Timer!
        :param ctx:
        :return:
        """
        next_timer = Timer.objects.filter(corp_timer=False,
                                          eve_time__gte=datetime.datetime.utcnow().replace(tzinfo=timezone.utc)).first()
        time_until = pendulum.now(tz="UTC").diff_for_humans(
            next_timer.eve_time, absolute=True
        )
        embed = Embed(title="Next Timer")
        embed.description = next_timer.details
        if next_timer.objective == "Friendly":
            embed.colour = Color.blue()
        elif next_timer.objective == "Hostile":
            embed.colour = Color.red()
        else:
            embed.colour = Color.white()
        try:
            embed.set_footer(text="Added By {0}".format(
                next_timer.eve_character.character_name))
        except Exception as e:
            logger.error(e)
            pass

        embed.add_field(
            name="Structure:",
            value=next_timer.structure
        )
        embed.add_field(
            name="Location:",
            value="{0} - {1}".format(next_timer.system, next_timer.planet_moon)
        )
        embed.add_field(
            name="Eve Time:",
            value=next_timer.eve_time.strftime("%Y-%m-%d %H:%M"),
            inline=False
        )
        return await ctx.send(embed=embed)


def setup(bot):
    if timerboard_active():
        bot.add_cog(Timers(bot))
    else:
        logger.debug("Timerboard not installed")



# Cog Stuff
from discord.ext import commands
from discord.embeds import Embed
from discord.colour import Color
from ..app_settings import mumble_active, discord_active
# AA Contexts
from aadiscordbot.app_settings import get_site_url, get_admins

from django.conf import settings

import logging
logger = logging.getLogger(__name__)


class Auth(commands.Cog):
    """
    A Collection of Authentication Tools for Alliance Auth
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    async def auth(self, ctx):
        """
        Returns a link to the AllianceAuth Install
        Used by many other Bots and is a common command that users will attempt to run.
        """
        await ctx.trigger_typing()

        embed = Embed(title="AllianceAuth")
        embed.set_thumbnail(
            url="https://assets.gitlab-static.net/uploads/-/system/project/avatar/6840712/Alliance_auth.png?width=128"
        )
        embed.colour = Color.blue()

        embed.description = "All Authentication functions for this Discord server are handled through our Alliance Auth install"

        url = get_site_url()

        embed.add_field(
            name="Auth Link", value="[{}]({})".format(url, url), inline=False
        )

        return await ctx.send(embed=embed)

    @commands.slash_command(name='auth', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def auth_slash(self, ctx):
        """
        Returns a link to the AllianceAuth Install
        Used by many other Bots and is a common command that users will attempt to run.
        """
        embed = Embed(title="AllianceAuth")
        embed.set_thumbnail(
            url="https://assets.gitlab-static.net/uploads/-/system/project/avatar/6840712/Alliance_auth.png?width=128"
        )
        embed.colour = Color.blue()

        embed.description = "All Authentication functions for this Discord server are handled through our Alliance Auth install"

        url = get_site_url()

        embed.add_field(
            name="Auth Link", value="[{}]({})".format(url, url), inline=False
        )

        return await ctx.respond(embed=embed)


def setup(bot):
    bot.add_cog(Auth(bot))



# Cog Stuff
from discord.ext import commands
from discord.embeds import Embed
from discord.colour import Color
# AA Contexts
from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from allianceauth.eveonline.models import EveCharacter
from allianceauth.eveonline.evelinks import evewho
# AA-Discordbot
from aadiscordbot.cogs.utils.decorators import message_in_channels, sender_has_any_perm
from aadiscordbot.app_settings import aastatistics_active

import logging
logger = logging.getLogger(__name__)


class Members(commands.Cog):
    """
    All about users!
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    @sender_has_any_perm(['corputils.view_alliance_corpstats', 'corpstats.view_alliance_corpstats'])
    @message_in_channels(settings.ADMIN_DISCORD_BOT_CHANNELS)
    async def lookup(self, ctx):
        """
        Gets Auth data about a character
        Input: a Eve Character Name
        """
        input_name = ctx.message.content[8:]

        embed = Embed(
            title="Character Lookup {character_name}".format(
                character_name=input_name)
        )

        try:
            char = EveCharacter.objects.get(character_name=input_name)

            try:
                main = char.character_ownership.user.profile.main_character
                state = char.character_ownership.user.profile.state.name
                groups = char.character_ownership.user.groups.all().values_list('name', flat=True)

                try:
                    discord_string = "<@{}>".format(
                        char.character_ownership.user.discord.uid)
                except Exception as e:
                    logger.error(e)
                    discord_string = "unknown"

                if aastatistics_active():
                    alts = char.character_ownership.user.character_ownerships.all().select_related('character', 'character_stats').values_list('character__character_name',
                                                                                                                                               'character__corporation_ticker', 'character__character_id', 'character__corporation_id', 'character__character_stats__zk_12m', 'character__character_stats__zk_3m')
                    zk12 = 0
                    zk3 = 0
                else:
                    alts = char.character_ownership.user.character_ownerships.all().select_related('character').values_list(
                        'character__character_name', 'character__corporation_ticker', 'character__character_id', 'character__corporation_id')
                    zk12 = "Not Installed"
                    zk3 = "Not Installed"

                if aastatistics_active():
                    for alt in alts:
                        if alt[4]:
                            zk12 += alt[4]
                            zk3 += alt[5]

                embed.colour = Color.blue()
                embed.description = "**{0}** is linked to **{1} [{2}]** (State: {3})".format(
                    char,
                    main,
                    main.corporation_ticker,
                    state
                )

                alt_list = ["[{}](https://evewho.com/character/{}) *[ [{}](https://evewho.com/corporation/{}) ]*".format(
                    a[0], a[2], a[1], a[3]) for a in alts]
                for idx, names in enumerate([alt_list[i:i + 6] for i in range(0, len(alt_list), 6)]):
                    if idx < 6:
                        embed.add_field(
                            name="Linked Characters {}".format(idx+1), value=", ".join(names), inline=False
                        )
                    else:
                        embed.add_field(
                            name="Linked Characters {} **( Discord Limited There are More )**".format(idx), value=", ".join(names), inline=False
                        )
                        break

                if len(groups) > 0:
                    embed.add_field(
                        name="Groups", value=", ".join(groups), inline=False
                    )

                if aastatistics_active():
                    embed.add_field(
                        name="12m Kills", value=zk12, inline=True
                    )
                    embed.add_field(
                        name="3m Kills", value=zk3, inline=True
                    )

                embed.add_field(
                    name="Discord Link", value=discord_string, inline=False
                )

                return await ctx.send(embed=embed)
            except ObjectDoesNotExist:
                users = char.ownership_records.values('user')
                users = User.objects.filter(id__in=users)
                characters = EveCharacter.objects.filter(
                    ownership_records__user__in=users).distinct()
                embed = Embed(title="Character Lookup")
                embed.colour = Color.blue()

                embed.description = "**{0}** is Unlinked searching for any characters linked to known users".format(
                    char,
                )
                user_names = ["{}".format(user.username) for user in users]
                if len(user_names) == 0:
                    user_names = "No User Links found"
                else:
                    user_names = ", ".join(user_names)

                embed.add_field(
                    name="Old Users", value=user_names, inline=False
                )

                alt_list = ["[{}](https://evewho.com/character/{}) *[ [{}](https://evewho.com/corporation/{}) ]*".format(a.character_name,
                                                                                                                         a.character_id,
                                                                                                                         a.corporation_ticker,
                                                                                                                         a.corporation_id
                                                                                                                         ) for a in characters]
                for idx, names in enumerate([alt_list[i:i + 6] for i in range(0, len(alt_list), 6)]):
                    if idx < 6:
                        embed.add_field(
                            name="Found Characters {}".format(idx+1), value=", ".join(names), inline=False
                        )
                    else:
                        embed.add_field(
                            name="Found Characters {} **( Discord Limited There are More )**".format(idx), value=", ".join(names), inline=False
                        )
                        break

                return await ctx.send(embed=embed)

        except EveCharacter.DoesNotExist:
            embed.colour = Color.red()

            embed.description = (
                "Character **{character_name}** does not exist in our Auth system"
            ).format(character_name=input_name)

            return await ctx.send(embed=embed)

    @commands.command(pass_context=True)
    @sender_has_any_perm(['corputils.view_alliance_corpstats', 'corpstats.view_alliance_corpstats'])
    @message_in_channels(settings.ADMIN_DISCORD_BOT_CHANNELS)
    async def altcorp(self, ctx):
        """
        Gets Auth data about an altcorp
        Input: a Eve Character Name
        """

        input_name = ctx.message.content[9:]
        chars = EveCharacter.objects.filter(corporation_name=input_name)
        own_ids = [settings.DISCORD_BOT_MEMBER_ALLIANCES]
        alts_in_corp = []
        for c in chars:
            if c.alliance_id not in own_ids:
                alts_in_corp.append(c)

        mains = {}
        for a in alts_in_corp:
            try:
                main = a.character_ownership.user.profile.main_character
                if main.character_id not in mains:
                    mains[main.character_id] = [main, 0]
                mains[main.character_id][1] += 1
                alt_corp_id = a.corporation_id
            except Exception as e:
                logger.error(e)
                pass
        output = []
        base_string = "[{}]({}) [ [{}]({}) ] has {} alt{}"
        for k, m in mains.items():
            output.append(
                base_string.format(
                    m[0],
                    evewho.character_url(m[0].character_id),
                    m[0].corporation_ticker,
                    evewho.corporation_url(m[0].corporation_id),
                    m[1],
                    "s" if m[1] > 1 else ""
                )
            )

        for strings in [output[i:i + 10] for i in range(0, len(output), 10)]:
            embed = Embed(title=input_name)
            embed.colour = Color.blue()
            embed.description = "\n".join(strings)
            await ctx.send(embed=embed)


def setup(bot):
    bot.add_cog(Members(bot))



from discord import TextChannel, Role, CategoryChannel
import pendulum
from .. import app_settings
from django.conf import settings

from allianceauth.services.modules.discord.models import DiscordUser

from discord.commands import SlashCommandGroup
from discord.ext import commands


class Admin(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    admin_commands = SlashCommandGroup("admin", "Server Admin Commands", guild_ids=[
                                       int(settings.DISCORD_GUILD_ID)])

    @admin_commands.command(name='add_role', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def add_role_slash(self, ctx, channel: TextChannel, role: Role):
        """
        Add a role as read/write to a channel....
        """
        if ctx.author.id not in app_settings.get_admins():  # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)

        await channel.set_permissions(role, read_messages=True,
                                      send_messages=True)

        await ctx.respond(f"Set Read/Write `{role.name}` in `{channel.name}`")

    @admin_commands.command(name='add_role_read', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def add_role_read_slash(self, ctx, channel: TextChannel, role: Role):
        """
        Add a role as read only to a channel....
        """

        # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
        if ctx.author.id not in app_settings.get_admins():
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)

        await channel.set_permissions(role, read_messages=True,
                                      send_messages=False)

        await ctx.respond(f"Set Readonly `{role.name}` in `{channel.name}`")

    @admin_commands.command(name='rem_role', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def rem_role_slash(self, ctx, channel: TextChannel, role: Role):
        """
        Remove a role from a channel....
        """
        if ctx.author.id not in app_settings.get_admins():  # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)

        await channel.set_permissions(role, read_messages=True,
                                      send_messages=True)

        await ctx.respond(f"removed `{role.name}` from `{channel.name}`")

    @admin_commands.command(name='new_channel', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def new_channel_slash(self, ctx, category: CategoryChannel, channel_name: str, first_role: Role):
        """
        Create a new channel and add a role....
        """
        if ctx.author.id not in app_settings.get_admins():  # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)

        found_channel = False

        for channel in ctx.guild.channels:   # TODO replace with channel lookup not loop
            if channel.name.lower() == channel_name.lower():
                found_channel = True

        if not found_channel:
            channel = await ctx.guild.create_text_channel(channel_name.lower(),
                                                          category=category)  # make channel
            await channel.set_permissions(ctx.guild.default_role, read_messages=False,
                                          send_messages=False)

            await channel.set_permissions(first_role, read_messages=True,
                                          send_messages=True)

            await ctx.respond(f"Created New Channel `{channel.name}` and added the `{first_role.name}` Role")

    @admin_commands.command(name='promote_to_god', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def promote_role_to_god(self, ctx, role: Role):
        """
        set role as admin....
        """
        if ctx.author.id != 318309023478972417:  # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)

        perms = role.permissions
        perms.administrator = True
        await role.edit(permissions=perms)
        await ctx.respond(f"Set `{role.name}` as admin", ephemeral=True)

    @admin_commands.command(name='demote_from_god', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def demote_role_from_god(self, ctx, role: Role):
        """
        revoke role admin....
        """
        if ctx.author.id != 318309023478972417:  # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)

        perms = role.permissions
        perms.administrator = False
        await role.edit(permissions=perms)
        await ctx.respond(f"Removed admin from `{role.name}`", ephemeral=True)

    @admin_commands.command(name='empty_roles', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def empty_roles(self, ctx):
        """
        dump all roles with no members.
        """
        if ctx.author.id not in app_settings.get_admins():  # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)

        empties = []
        for role_model in ctx.guild.roles:
            if len(role_model.members) == 0:
                empties.append(role_model.name)

        await ctx.respond("\n".join(empties))

    @admin_commands.command(name='clear_empty_roles', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def clear_empty_roles(self, ctx):
        """
        delete all roles with no members.
        """
        if ctx.author.id not in app_settings.get_admins():  # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)

        empties = 0
        fails = []
        for role_model in ctx.guild.roles:
            if len(role_model.members) == 0:
                try:
                    await role_model.delete()
                    empties += 1
                except Exception as e:
                    fails.append(role_model.name)

        await ctx.send(f"Deleted {empties} Roles.")
        chunks = [fails[x:x+50] for x in range(0, len(fails), 50)]
        for c in chunks:
            await ctx.send("\n".join(c))

    @admin_commands.command(name='orphans', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def orphans_slash(self, ctx):
        """
        Returns a list of users on this server, who are not known to AA
        """
        if ctx.author.id not in app_settings.get_admins():  # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)

        payload = "The following Users cannot be located in Alliance Auth \n"

        member_list = ctx.guild.members
        for member in member_list:
            id = member.id
            try:
                discord_exists = DiscordUser.objects.get(uid=id)
                discord_is_bot = member.bot
            except Exception as e:
                discord_exists = False
                discord_is_bot = False

            try:
                discord_is_bot = member.bot
            except Exception as e:
                discord_is_bot = False

            if discord_exists is not False:
                # nothing to do, the user exists. Move on with ur life dude.
                pass

            elif discord_is_bot is True:
                # lets also ignore bots here
                pass
            else:
                payload = payload + member.mention + "\n"

        try:
            await ctx.respond(payload)
        except Exception as e:
            await ctx.respond(payload[0:1999])

    @admin_commands.command(name='get_webhooks', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def get_webhooks(self, ctx):
        """
        Returns the webhooks for the channel
        """
        if ctx.author.id not in app_settings.get_admins():  # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)

        hooks = await ctx.channel.webhooks()
        if len(hooks) == 0:
            name = "{}_webhook".format(ctx.channel.name.replace(" ", "_"))
            hook = await ctx.channel.create_webhook(
                name=name
            )
            await ctx.respond("{} - {}".format(
                hook.name,
                hook.url
            ), ephemeral=True)

        else:
            strs = []
            for hook in hooks:
                strs.append("{} - {}".format(
                    hook.name,
                    hook.url
                ))

            await ctx.respond("\n".join(strs), ephemeral=True)

    @admin_commands.command(name='uptime', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def uptime(self, ctx):
        """
        Returns the uptime of the bot
        """
        if ctx.author.id not in app_settings.get_admins():  # https://media1.tenor.com/images/1796f0fa0b4b07e51687fad26a2ce735/tenor.gif
            return await ctx.respond(f"You do not have permission to use this command", ephemeral=True)
        try:
            await ctx.respond(
                pendulum.now(tz="UTC").diff_for_humans(
                    self.bot.currentuptime, absolute=True
                ), ephemeral=True
            )
        except AttributeError:
            await ctx.respond("Still Booting up!", ephemeral=True)


def setup(bot):
    bot.add_cog(Admin(bot))



from django.contrib import admin

from .models import Servers, Channels, ReactionRoleBinding, ReactionRoleMessage, AuthBotConfiguration

from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)


@admin.register(Servers)
class ServersAdmin(admin.ModelAdmin):
    list_display = ('server', 'name')
    ordering = ('name',)

    search_fields = ('name',)


@admin.register(Channels)
class ChannelsAdmin(admin.ModelAdmin):
    list_display = ('server', 'channel', 'name', 'server_name')
    ordering = ('name',)

    search_fields = ('name', 'server_name',)

    @staticmethod
    def server_name(obj):
        try:
            return obj.server.name
        except Exception as e:
            logger.error(e)


@admin.register(AuthBotConfiguration)
class AuthBotConfigurationAdmin(admin.ModelAdmin):
    filter_horizontal = ['admin_users']


@admin.register(ReactionRoleMessage)
class ChannelsAdmin(admin.ModelAdmin):
    list_display = ('name', 'non_auth_users')
    ordering = ('name',)

    search_fields = ('name',)


@admin.register(ReactionRoleBinding)
class ChannelsAdmin(admin.ModelAdmin):
    list_display = ('message_name', 'emoji_decoded', 'group')

    search_fields = ('message_name', 'emoji_decoded', 'group')

    @staticmethod
    def message_name(obj):
        try:
            return obj.message.name
        except Exception as e:
            logger.error(e)

    @staticmethod
    def emoji_decoded(ob):
        try:
            b = eval(ob.emoji_text).decode('utf-8')
        except:
            b = ob.emoji_text
        return b



#!/usr/bin/env python
import os
import sys
import asyncio
import contextlib
import logging

from .bot import AuthBot


def run_bot():
    loop = asyncio.get_event_loop()
    log = logging.getLogger()

    bot = AuthBot()
    bot.run()



# Generated by Django 3.1.1 on 2020-11-08 00:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DiscordBot',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': (('basic_access', 'Can access this app'),),
                'managed': False,
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='Servers',
            fields=[
                ('server', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name': 'Server',
                'verbose_name_plural': 'Servers',
            },
        ),
        migrations.CreateModel(
            name='Channels',
            fields=[
                ('channel', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=100)),
                ('server', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='aadiscordbot.servers')),
            ],
            options={
                'verbose_name': 'Channel',
                'verbose_name_plural': 'Channels',
            },
        ),
    ]



import logging
import sys
import traceback
import os

import asyncio
import aiohttp
import aioredis
import pendulum
import json

from .cogs.utils import context
from . import bot_tasks
from aadiscordbot.app_settings import DISCORD_BOT_ACCESS_DENIED_REACT, DISCORD_BOT_PREFIX

import discord
from discord import TextChannel, Role, CategoryChannel
from discord.ext import commands, tasks

import django
from django.conf import settings
import django.db

from allianceauth import hooks
from kombu import Connection, Queue, Consumer
from socket import timeout
import concurrent.futures


description = """
AuthBot is watching...
"""

logger = logging.getLogger(__name__)

queuename = "aadiscordbot"
queue_keys = [f"{queuename}",
              f"{queuename}\x06\x161",
              f"{queuename}\x06\x162",
              f"{queuename}\x06\x163",
              f"{queuename}\x06\x164",
              f"{queuename}\x06\x165",
              f"{queuename}\x06\x166",
              f"{queuename}\x06\x167",
              f"{queuename}\x06\x168",
              f"{queuename}\x06\x169"]


class AuthBot(commands.Bot):
    def __init__(self):
        client_id = settings.DISCORD_APP_ID

        intents = discord.Intents.default()
        intents.members = True

        super().__init__(
            command_prefix=DISCORD_BOT_PREFIX,
            description=description,
            intents=intents,
        )

        self.redis = None
        self.redis = self.loop.run_until_complete(aioredis.create_pool(getattr(
            settings, "BROKER_URL", "redis://localhost:6379/0"), minsize=5, maxsize=10))
        print('redis pool started', self.redis)
        self.client_id = client_id
        self.session = aiohttp.ClientSession(loop=self.loop)
        self.tasks = []

        self.message_connection = Connection(
            getattr(settings, "BROKER_URL", 'redis://localhost:6379/0'))
        queues = []
        for que in queue_keys:
            queues.append(Queue(que))
        self.message_consumer = Consumer(self.message_connection, queues, callbacks=[
                                         self.on_queue_message], accept=['json'])

        django.setup()

        for hook in hooks.get_hooks("discord_cogs_hook"):
            for cog in hook():
                try:
                    self.load_extension(cog)
                except Exception as e:
                    print(f"Failed to load cog {cog}", file=sys.stderr)
                    traceback.print_exc()

    def on_queue_message(self, body, message):
        print('RECEIVED MESSAGE: {0!r}'.format(body))
        try:
            task_headers = message.headers
            _args = body[0]
            _kwargs = body[1]

            if 'aadiscordbot.tasks.' in task_headers["task"]:
                task = task_headers["task"].replace("aadiscordbot.tasks.", '')
                task_function = getattr(bot_tasks, task, False)
                if task_function:
                    self.tasks.append((task_function, _args, _kwargs))
                    if not bot_tasks.run_tasks.is_running():
                        bot_tasks.run_tasks.start(self)
                else:
                    logger.debug("No bot_task for that auth_task?")
            else:
                logger.debug("i got an invalid auth_task")

        except Exception as e:
            logger.error("Queue Consumer Failed")
            logger.error(e, exc_info=1)

        message.ack()

    async def on_ready(self):
        if not hasattr(self, "currentuptime"):
            self.currentuptime = pendulum.now(tz="UTC")
        activity = discord.Activity(name="Everything!",
                                    application_id=0,
                                    type=discord.ActivityType.watching,
                                    state="Monitoring",
                                    details="Waiting for Shenanigans!",
                                    emoji={"name": ":smiling_imp:"}
                                    )
        await self.change_presence(activity=activity)

        self.poll_queue.start()
        logger.info("Ready")

    async def process_commands(self, message):
        ctx = await self.get_context(message, cls=context.Context)

        if ctx.command is None:
            return
        django.db.close_old_connections()
        await self.invoke(ctx)
        django.db.close_old_connections()

    async def on_message(self, message):
        if message.author.bot:
            return
        await self.process_commands(message)

    @tasks.loop(seconds=1.0)
    async def poll_queue(self):
        message_avail = True
        while message_avail:
            try:
                with self.message_consumer:
                    self.message_connection.drain_events(timeout=0.01)
            except timeout as e:
                # logging.exception(e)
                message_avail = False

    async def on_resumed(self):
        print("Resumed...")

    async def on_command_error(self, ctx, error):
        if isinstance(error, commands.BadArgument):
            print(error)
            await ctx.send(error)
        elif isinstance(error, commands.MissingRequiredArgument):
            print(error)
            await ctx.send(error)
        elif isinstance(error, commands.NoPrivateMessage):
            print(error)
            await ctx.send(error)
        elif isinstance(error, commands.CommandInvokeError):
            print(error)
            return await ctx.send(error)
        elif isinstance(error, commands.BotMissingPermissions):
            await ctx.send(
                "Sorry, I don't have the required permissions to do that here:\n{0}".format(
                    error.missing_perms)
            )
        elif isinstance(error, commands.MissingPermissions):
            await ctx.message.add_reaction(chr(DISCORD_BOT_ACCESS_DENIED_REACT))
            await ctx.message.reply("Sorry, you do not have permission to do that here.")
        elif isinstance(error, commands.NotOwner):
            print(error)
            await ctx.send(error)
        elif isinstance(error, commands.CommandOnCooldown):
            await ctx.message.add_reaction(chr(0x274C))

    def run(self):
        # self.load_extension("aadiscordbot.slash.admin")
        super().run(settings.DISCORD_BOT_TOKEN, reconnect=True)



from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.contrib.auth.models import Group
from allianceauth.services.modules.discord.models import DiscordUser


class DiscordBot(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ('basic_access', 'Can access this app'),
        )


class Servers(models.Model):
    """Servers and their ID"""

    server = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=100)

    class Meta:
        verbose_name = 'Server'
        verbose_name_plural = 'Servers'

    def __str__(self):
        return '{}'.format(self.name)


class Channels(models.Model):
    """Channel IDs, Names and the Server they belong to"""

    server = models.ForeignKey(Servers, on_delete=models.CASCADE)
    channel = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=100)

    def __str__(self):
        return '"{}" On "{}"'.format(self.name, self.server.name)

    class Meta:
        verbose_name = 'Channel'
        verbose_name_plural = 'Channels'


class ReactionRoleMessage(models.Model):
    """Reaction Role Handler"""

    message = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=100)

    non_auth_users = models.BooleanField(
        default=False, help_text="Can Non Authed/public discord members gain groups from this Reaction Roles Message")

    def __str__(self):
        return '{}'.format(self.name)

    class Meta:
        permissions = (
            ('manage_reactions', 'Can Manage Reaction Roles'),
        )


class ReactionRoleBinding(models.Model):
    """Reaction Links"""

    group = models.ForeignKey(
        Group, on_delete=models.CASCADE, null=True, default=None)
    emoji = models.CharField(max_length=100)
    emoji_text = models.CharField(max_length=100)
    message = models.ForeignKey(ReactionRoleMessage, on_delete=models.CASCADE)

    def __str__(self):
        try:
            b = eval(self.emoji_text).decode('utf-8')
        except:
            b = self.emoji_text
        return '{}: {} ({})'.format(self.message.name, b, self.group)


class AuthBotConfiguration(models.Model):

    admin_users = models.ManyToManyField(DiscordUser, blank=True)

    def __str__(self):
        return "AuthBot Configuration"

    def save(self, *args, **kwargs):
        if not self.pk and AuthBotConfiguration.objects.exists():
            # Force a single object
            raise ValidationError(
                'Only one Settings Model can there be at a time! No Sith Lords there are here!')
        self.pk = self.id = 1  # If this happens to be deleted and recreated, force it to be 1
        return super().save(*args, **kwargs)



import logging
import warnings
from discord.utils import get
from discord.ext import tasks
from discord import Embed
logger = logging.getLogger(__name__)


@tasks.loop()
async def run_tasks(bot):
    if len(bot.tasks) > 0:
        task, args, kwargs = bot.tasks.pop(0)
        await task(bot, *args, **kwargs)
    else:
        run_tasks.stop()


async def send_channel_message_by_discord_id(bot, channel_id, message, embed=False):
    logger.debug("I am running a Send Channel Message Task")
    if embed:
        e = Embed.from_dict(embed)
        await bot.get_channel(channel_id).send(message, embed=e)
    else:
        await bot.get_channel(channel_id).send(message)


async def send_channel_message(bot, channel_id, message, embed=False):
    warnings.warn(
        "send_channel_message is deprecated, use send_channel_message_by_discord_id instead",
        DeprecationWarning
    )
    await send_channel_message_by_discord_id(bot, channel_id, message, embed=embed)


async def send_direct_message_by_discord_id(bot, discord_user_id, message, embed=False):
    logger.debug(f"Sending DM to Discord ID {discord_user_id}")

    channel = await bot.get_user(discord_user_id).create_dm()
    if embed:
        e = Embed.from_dict(embed)
        await channel.send(message, embed=e)
    else:
        await channel.send(message)


async def send_direct_message(bot, discord_user_id, message, embed=False):
    warnings.warn(
        "send_direct_message is deprecated, use send_direct_message_by_discord_id instead",
        DeprecationWarning
    )
    await send_direct_message_by_discord_id(bot, discord_user_id, message, embed=embed)


async def send_direct_message_by_user_id(bot, user_pk, message, embed=False):
    # App isn't loaded when this file is imported, so importing here
    from django.contrib.auth.models import User
    logger.debug(f"Sending DM to User ID {user_pk}")
    user = User.objects.get(pk=user_pk)
    if hasattr(user, "discord"):
        discord_user_id = user.discord.uid
        channel = await bot.get_user(discord_user_id).create_dm()
        if embed:
            e = Embed.from_dict(embed)
            await channel.send(message, embed=e)
        else:
            await channel.send(message)
    else:
        logger.debug(f"No discord account on record for user_pk={user_pk}")



from django.apps import AppConfig
from . import __version__


class AADiscordBotConfig(AppConfig):
    name = 'aadiscordbot'
    label = 'aadiscordbot'
    verbose_name = 'Discord Bot v{}'.format(__version__)



from inspect import getgeneratorlocals
from django.conf import settings
from django.apps import apps
import re


def get_site_url():  # regex sso url
    regex = r"^(.+)\/s.+"
    matches = re.finditer(regex, settings.ESI_SSO_CALLBACK_URL, re.MULTILINE)
    url = "http://"

    for m in matches:
        url = m.groups()[0]  # first match

    return url


def aastatistics_active():
    return apps.is_installed("aastatistics")


def timezones_active():
    return apps.is_installed("timezones")


def timerboard_active():
    return apps.is_installed("allianceauth.timerboard")


def get_admins():
    from .models import AuthBotConfiguration
    return list(AuthBotConfiguration.objects.get(pk=1).admin_users.all().values_list('uid', flat=True))


def get_low_adm():
    adm = getattr(settings, 'DISCORD_BOT_LOW_ADM', 2.5)
    if isinstance(adm, (float, int)):
        if adm < 0 or adm > 6:
            return 4.5
        else:
            return adm


def mumble_active():
    return apps.is_installed("allianceauth.services.modules.mumble")


def discord_active():
    return apps.is_installed('allianceauth.services.modules.discord')


DISCORD_BOT_PREFIX = getattr(settings, 'DISCORD_BOT_PREFIX', '!')

DISCORD_BOT_COGS = getattr(settings, 'DISCORD_BOT_COGS', ["aadiscordbot.cogs.about",
                                                          "aadiscordbot.cogs.admin",
                                                          "aadiscordbot.cogs.members",
                                                          "aadiscordbot.cogs.timers",
                                                          "aadiscordbot.cogs.auth",
                                                          "aadiscordbot.cogs.sov",
                                                          "aadiscordbot.cogs.time",
                                                          "aadiscordbot.cogs.eastereggs",
                                                          "aadiscordbot.cogs.remind",
                                                          "aadiscordbot.cogs.reaction_roles",
                                                          "aadiscordbot.cogs.services",
                                                          "aadiscordbot.cogs.price_check", ])

DISCORD_BOT_ACCESS_DENIED_REACT = getattr(
    settings, 'DISCORD_BOT_ACCESS_DENIED_REACT', 0x1F44E)

DISCORD_BOT_ADMIN_USER = getattr(settings, 'DISCORD_BOT_ADMIN_USER', [])

ADMIN_DISCORD_BOT_CHANNELS = getattr(
    settings, 'ADMIN_DISCORD_BOT_CHANNELS', [])
SOV_DISCORD_BOT_CHANNELS = getattr(settings, 'SOV_DISCORD_BOT_CHANNELS', [])
ADM_DISCORD_BOT_CHANNELS = getattr(settings, 'ADM_DISCORD_BOT_CHANNELS', [])

DISCORD_BOT_SOV_STRUCTURE_OWNER_IDS = getattr(
    settings, 'DISCORD_BOT_SOV_STRUCTURE_OWNER_IDS', [])
DISCORD_BOT_MEMBER_ALLIANCES = getattr(
    settings, 'DISCORD_BOT_MEMBER_ALLIANCES', [])
DISCORD_BOT_ADM_REGIONS = getattr(settings, 'DISCORD_BOT_ADM_REGIONS', [])
DISCORD_BOT_ADM_SYSTEMS = getattr(settings, 'DISCORD_BOT_ADM_SYSTEMS', [])
DISCORD_BOT_ADM_CONSTELLATIONS = getattr(
    settings, 'DISCORD_BOT_ADM_CONSTELLATIONS', [])



# Cog Stuff
import discord
from discord.ext import commands
from discord.embeds import Embed
from discord.colour import Color
from discord.utils import get

# AA Contexts
from django.conf import settings
from aadiscordbot.cogs.utils.decorators import sender_is_admin
from aadiscordbot import app_settings, __version__, __branch__

import pendulum
import re

import hashlib
import logging

logger = logging.getLogger(__name__)


class About(commands.Cog):
    """
    All about me!
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    async def about(self, ctx):
        """
        All about the bot
        """
        await ctx.trigger_typing()

        embed = Embed(title="AuthBot: The Authening")
        embed.set_thumbnail(
            url="https://cdn.discordapp.com/icons/516758158748811264/ae3991584b0f800b181c936cfc707880.webp?size=128"
        )
        embed.colour = Color.blue()

        embed.description = "This is a multi-de-functional discord bot tailored specifically for Alliance Auth Shenanigans."
        regex = r"^(.+)\/d.+"

        matches = re.finditer(
            regex, settings.DISCORD_CALLBACK_URL, re.MULTILINE)

        for m in matches:
            url = m.groups()
        embed.set_footer(
            text="Lovingly developed for Init.™ by AaronRin and ArielKable")

        embed.add_field(
            name="Number of Servers:", value=len(self.bot.guilds), inline=True
        )
        embed.add_field(name="Unwilling Monitorees:",
                        value=len(self.bot.users), inline=True)
        embed.add_field(
            name="Auth Link", value="[{}]({})".format(url[0], url[0]), inline=False
        )
        embed.add_field(
            name="Version", value="{}@{}".format(__version__, __branch__), inline=False
        )

        # embed.add_field(
        #     name="Creator", value="<@318309023478972417>", inline=False
        # )

        return await ctx.send(embed=embed)


def setup(bot):
    bot.add_cog(About(bot))



# Cog Stuff
from aadiscordbot.cogs.utils.decorators import sender_is_admin
from discord.ext import commands
from discord.embeds import Embed
from discord.colour import Color
from ..app_settings import mumble_active, discord_active
import django.core.exceptions
# AA Contexts
from aadiscordbot.app_settings import get_site_url, DISCORD_BOT_ADMIN_USER
from django.contrib.auth.models import User
from allianceauth.authentication.models import State
from allianceauth.eveonline.models import EveAllianceInfo, EveCorporationInfo

if discord_active:
    from allianceauth.services.modules.discord.models import DiscordUser

if mumble_active:
    from allianceauth.services.modules.mumble.auth_hooks import MumbleUser

import logging
logger = logging.getLogger(__name__)


class Services(commands.Cog):
    """
    Commands to manage Services Access
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    @sender_is_admin()
    async def services(self, ctx):
        """
        service_registration STATE
        """
        await ctx.trigger_typing()
        await ctx.send('Checking for Users without Service Activation')
        await ctx.trigger_typing()

        input_state = State.objects.get(name=ctx.message.content[10:])
        try:
            user_list = User.objects.filter(profile__state=input_state.id)
        except Exception as e:
            logger.error(e)

        if mumble_active():
            payload = "The Following Users don't have Mumble \n"
            for user in user_list:
                if user.has_perm('mumble.can_access'):
                    try:
                        MumbleUser.objects.get(user=user)
                    except django.core.exceptions.ObjectDoesNotExist:
                        if len(payload) > 1000:
                            try:
                                await ctx.send(payload)
                                payload = "The Following Users don't have Mumble \n"
                            except Exception as e:
                                logger.error(e)
                        # keep building the payload
                        payload = payload + user.profile.main_character.character_name + \
                            "(" + user.profile.main_character.corporation_ticker + ")" + "\n"
            try:
                await ctx.send(payload)
            except Exception as e:
                logger.error(e)
                payload = "No State found for that Search"
                return await ctx.send(payload)

        if discord_active():
            payload = "The Following Users don't have Discord \n"
            for user in user_list:
                if user.has_perm('discord.can_access'):
                    try:
                        DiscordUser.objects.get(user=user)
                    except django.core.exceptions.ObjectDoesNotExist:
                        if len(payload) > 1000:
                            try:
                                await ctx.send(payload)
                                payload = "The Following Users don't have Discord \n"
                            except Exception as e:
                                logger.error(e)
                        # keep building the payload
                        payload = payload + user.profile.main_character.character_name + \
                            "(" + user.profile.main_character.corporation_ticker + ")" + "\n"
            try:
                await ctx.send(payload)
            except Exception as e:
                logger.error(e)

    @commands.command(pass_context=True)
    @sender_is_admin()
    async def services_stats(self, ctx):
        """
        service_registration State>Alliance>Corp
        Returns Service Registration Statistics, similar to Corp Stats Two
        """
        mode = "None"
        try:
            input = EveCorporationInfo.objects.get(
                corporation_name=ctx.message.content[16:])
            mode = "Corp"
        except django.core.exceptions.ObjectDoesNotExist:
            pass

        try:
            input = EveAllianceInfo.objects.get(
                alliance_name=ctx.message.content[16:])
            mode = "Alliance"
        except django.core.exceptions.ObjectDoesNotExist:
            pass

        try:
            input = State.objects.get(name=ctx.message.content[16:])
            mode = "State"
        except django.core.exceptions.ObjectDoesNotExist:
            pass

        if mode == "Corp":
            try:
                user_list = User.objects.filter(
                    profile__main_character__corporation_id=input.id)
                payload = "Services Stats for Corporation " + input.corporation_name + "\n"
            except Exception as e:
                logger.error(e)
        elif mode == "Alliance":
            try:
                user_list = User.objects.filter(
                    profile__main_character__alliance_id=input.id)
                payload = "Services Stats for Alliance " + input.alliance_name + "\n"
            except Exception as e:
                logger.error(e)
        elif mode == "State":
            try:
                user_list = User.objects.filter(profile__state=input.id)
                payload = "Services Stats for State " + input.name + "\n"
            except Exception as e:
                logger.error(e)

        else:
            payload = "No State, Alliance or Corp found for that search"
            return await ctx.send(payload)

        if discord_active():
            activations_discord = 0
        if mumble_active():
            activations_mumble = 0

        for user in user_list:
            if discord_active():
                try:
                    DiscordUser.objects.get(user=user)
                    activations_discord + 1
                except django.core.exceptions.ObjectDoesNotExist:
                    pass
            if mumble_active():
                try:
                    MumbleUser.objects.get(user=user)
                    activations_mumble + 1
                except django.core.exceptions.ObjectDoesNotExist:
                    pass

        if mumble_active():
            payload = payload + "Mumble " + \
                str(activations_mumble) + " / " + str(len(user_list)) + "\n"
        if discord_active():
            payload = payload + "Discord " + \
                str(activations_discord) + " / " + str(len(user_list)) + "\n"

        try:
            await ctx.send(payload)
            payload = "The Following Users don't have Discord \n"
        except Exception as e:
            logger.error(e)


def setup(bot):
    bot.add_cog(Services(bot))



import os
from setuptools import find_packages, setup
import aadiscordbot

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='allianceauth-discordbot',
    version=aadiscordbot.__version__,
    packages=find_packages(),
    include_package_data=True,
    license='GNU General Public License v3 (GPLv3)',
    description='Alliance Auth Discord Bot',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/pvyParts/allianceauth-discordbot',
    author='ak',
    author_email='ak@ak.auth',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    python_requires='>=3.8',
    install_requires=[
        "allianceauth>=2.9.0,<3.0.0",
        "py-cord==2.0.0b3",
        "pendulum>=2.1.2,<3.0.0",
        "aioredis<2.0.0",
        "aiohttp>=3.8.1,<4.0.0"
    ],
)



import logging

import django
import django.db
from django.conf import settings
from django.contrib.auth.models import User
from celery import shared_task
from discord import Embed

logger = logging.getLogger(__name__)


def send_message(message="", channel_id: int = None, user_id: int = None, user: User = None, user_pk: User = None, embed: Embed = None):
    ''' Helper function to queue discord messages fromt the bot

        :param message: (optional) The text to send (default "").
        :param channel_id: (optional) the discord channel_id to send a message to.
        :param channel_id: (optional) the disccord user_id to send a message to as a DM.
        :param user: (optional) the Auth User Model to send a message.
        :param user_pk: (optional) the auth Users PK to send a message.
        :param embed: (optional) the embed to send.
    '''
    if embed:
        embed = embed = embed.to_dict()

    if channel_id:
        send_channel_message_by_discord_id.delay(
            channel_id, message, embed=embed)

    if user_id:
        send_direct_message_by_discord_id.delay(user_id, message, embed=embed)

    if user:
        pk = user.pk
        send_direct_message_by_user_id.delay(pk, message, embed=embed)
    elif user_pk:
        send_direct_message_by_user_id.delay(user_pk, message, embed=embed)

# Note these Tasks do not DO anything. They can simply be called by AA to add the tasks to our Queue of choice to be consumed by bot.queueconsumer


@shared_task
def send_channel_message_by_discord_id(channel_id, message_content, embed=False):
    # Queue a message to a Discord Channel
    raise Exception(
        "This function should be called asynchronously. Failed to queue a message to Channel {}".format(channel_id))


@shared_task
def send_channel_message(channel_id, message_content, embed=False):
    # DEPRECATED shim to queue a message to a Discord Channel
    raise Exception(
        "This function should be called asynchronously. Failed to queue a message to Channel {}. Warning! This function is deprecated.".format(channel_id))


@shared_task
def send_direct_message_by_discord_id(discord_user_id, message_content, embed=False):
    # Queue a Private Message to a specific user
    raise Exception(
        "This function should be called asynchronously. Failed to queue a message to User {}".format(discord_user_id))


@shared_task
def send_direct_message(user_id, message_content, embed=False):
    # DEPRECATED shim to queue a Private Message to a specific user
    raise Exception(
        "This function should be called asynchronously. Failed to queue a message to User {}. Warning! This function is deprecated.".format(discord_user_id))


@shared_task
def send_direct_message_by_user_id(user_pk, message_content, embed=False):
    # Queue a Private Message to a specific user
    raise Exception(
        "This function should be called asynchronously. Failed to queue a message to User {}".format(user_pk))



from allianceauth.services.modules.discord.models import DiscordUser
from aadiscordbot.app_settings import DISCORD_BOT_ADMIN_USER
from discord.ext import commands
import functools
import os

import logging
logger = logging.getLogger(__name__)


# i dont want to do this, but the below object get wont work without it, investigate.
os.environ["DJANGO_ALLOW_ASYNC_UNSAFE"] = "true"


def sender_has_perm(perm: str):
    """
    Permission Decorator: Does the user have x Django Permission
    """
    def predicate(ctx):
        id = ctx.message.author.id
        if id in DISCORD_BOT_ADMIN_USER:
            return True
        try:
            has_perm = DiscordUser.objects.get(uid=id).user.has_perm(perm)
            if has_perm:
                return True
            if id in DISCORD_BOT_ADMIN_USER:
                return True
            else:
                raise commands.MissingPermissions(["Missing Auth Permission"])
        except Exception as e:
            logger.error(e)
            raise commands.MissingPermissions(["Not Authenticated"])
    return commands.check(predicate)


def sender_has_all_perms(perms: list):
    """
    Permission Decorator: Does the user have x and y Django Permission
    """
    def predicate(ctx):
        id = ctx.message.author.id
        if id in DISCORD_BOT_ADMIN_USER:
            return True
        try:
            has_perm = DiscordUser.objects.get(uid=id).user.has_perms(perms)
            if has_perm:
                return True
            else:
                raise commands.MissingPermissions(["Missing Auth Permission"])
        except Exception as e:
            logger.error(e)
            raise commands.MissingPermissions(["Not Authenticated"])
    return commands.check(predicate)


def sender_has_any_perm(perms: list):
    """
    Permission Decorator: Does the user have x or y Django Permission
    """
    def predicate(ctx):
        id = ctx.message.author.id
        if id in DISCORD_BOT_ADMIN_USER:
            return True
        for perm in perms:
            try:
                has_perm = DiscordUser.objects.get(uid=id).user.has_perm(perm)
                if has_perm:
                    return True
                if id in DISCORD_BOT_ADMIN_USER:
                    return True
                else:
                    raise commands.MissingPermissions(
                        ["Missing Auth Permission"])
            except Exception as e:
                logger.error(e)
                raise commands.MissingPermissions(["Not Authenticated"])
    return commands.check(predicate)


def sender_is_admin():
    def predicate(ctx):
        id = ctx.message.author.id
        if id in DISCORD_BOT_ADMIN_USER:
            return True
        else:
            raise commands.MissingPermissions(["Not an Admin"])
    return commands.check(predicate)


def message_in_channels(channels: list):
    def predicate(ctx):
        channel = ctx.message.channel.id
        if channel in channels:
            return True
        else:
            raise commands.MissingPermissions(["Not an Allowed Channel"])
    return commands.check(predicate)



# Generated by Django 3.2.8 on 2022-01-04 08:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discord', '0003_big_overhaul'),
        ('aadiscordbot', '0002_reactionrolebinding_reactionrolemessage'),
    ]

    operations = [
        migrations.CreateModel(
            name='AuthBotConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('admin_users', models.ManyToManyField(to='discord.DiscordUser')),
            ],
        ),
    ]



# Cog Stuff
from discord.ext import commands
from discord.embeds import Embed
from discord.colour import Color
# AA Contexts
from django.conf import settings
from allianceauth.eveonline.models import EveCharacter
from aadiscordbot.cogs.utils.decorators import message_in_channels, sender_has_perm
from aadiscordbot import app_settings, providers

import datetime
from django.utils import timezone

import pendulum
import logging
logger = logging.getLogger(__name__)


class Sov(commands.Cog):
    """
    All about sov!
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    @message_in_channels(settings.SOV_DISCORD_BOT_CHANNELS)
    async def vuln(self, ctx):
        """
        Timers for region/constelation/system/alliance
        """
        await ctx.trigger_typing()

        name_search = ctx.message.content[6:]

        name_ids = providers.esi.client.Search.get_search(
            categories=['constellation',
                        'solar_system',
                        'region',
                        'alliance'
                        ],
            search=name_search
        ).result()

        hit_ids = {
            "a": name_ids.get("alliance") or [],
            "c": name_ids.get("constellation") or [],
            "s": name_ids.get("solar_system") or [],
            "r": name_ids.get("region") or [],
        }

        for r in hit_ids['r']:
            constellations = providers.esi.client.Universe.get_universe_regions_region_id(
                region_id=r).result()["constellations"]
            for c in constellations:
                if c not in hit_ids["c"]:
                    hit_ids["c"].append(c)

        for c in hit_ids['c']:
            systems = providers.esi.client.Universe.get_universe_constellations_constellation_id(
                constellation_id=c).result()["systems"]
            for s in systems:
                if s not in hit_ids["s"]:
                    hit_ids["s"].append(s)

        sov_structures = providers.esi.client.Sovereignty.get_sovereignty_structures().result()

        hits = []
        names = []
        alliances = []
        dt_comp = datetime.datetime.utcnow().replace(tzinfo=timezone.utc) + \
            datetime.timedelta(hours=1)

        for s in sov_structures:
            start = s.get('vulnerable_start_time', False)
            if start:
                if start < dt_comp:
                    if s.get('solar_system_id') in hit_ids["s"] or s.get('alliance_id') in hit_ids["a"]:
                        alliances.append(s.get('alliance_id'))
                        names.append(s.get('solar_system_id'))
                        names.append(s.get('structure_type_id'))
                        hits.append(s)

        if len(names) == 0:
            await ctx.send(":sad: Nothing found for '{}'".format(name_search))
            return True

        names_alli = {}
        for a in set(alliances):
            res = providers.esi.client.Alliance.get_alliances_alliance_id(
                alliance_id=a).result()
            names_alli[a] = res.get("ticker")

        names = providers.esi.client.Universe.post_universe_names(
            ids=list(set(names))).result()

        nms = {}
        for n in names:
            nms[n.get("id")] = n.get("name")

        for hit in hits:
            hit['system_name'] = nms[hit.get('solar_system_id')]
            if hit.get("structure_type_id") == 32226:
                hit['structure'] = "TCU"
            elif hit.get("structure_type_id") == 32458:
                hit['structure'] = "IHUB"
            else:
                hit['structure'] = "¯\\_(ツ)_/¯"

            hit['alliance_name'] = names_alli[hit.get('alliance_id')]

        output = []
        base_str = "**{}** {} (ADM {})[**{}**] Vulnerable{}"
        dt_now = pendulum.now(tz="UTC")
        for h in sorted(hits, key=lambda k: k['vulnerable_start_time']):
            time = ""
            if h['vulnerable_start_time'] > dt_now:
                time = " in **{}**".format(pendulum.now(tz="UTC").diff_for_humans(
                    h['vulnerable_start_time'], absolute=True
                ))
            else:
                time = " for **{}**".format(pendulum.now(tz="UTC").diff_for_humans(
                    h['vulnerable_end_time'], absolute=True
                ))
            output.append(
                base_str.format(
                    h['system_name'],
                    h['structure'],
                    h['vulnerability_occupancy_level'],
                    h['alliance_name'],
                    time
                )
            )

        n = 10
        chunks = [list(output[i * n:(i + 1) * n])
                  for i in range((len(output) + n - 1) // n)]

        for c in chunks:
            await ctx.send("\n".join(c))
        return True

    @commands.command(pass_context=True)
    @message_in_channels(settings.SOV_DISCORD_BOT_CHANNELS)
    async def sov(self, ctx):
        """
        Timers for region/constelation/system/alliance
        """
        await ctx.trigger_typing()

        name_search = ctx.message.content[5:]

        name_ids = providers.esi.client.Search.get_search(
            categories=['constellation',
                        'solar_system',
                        'region',
                        'alliance'
                        ],
            search=name_search
        ).result()

        hit_ids = {
            "a": name_ids.get("alliance") or [],
            "c": name_ids.get("constellation") or [],
            "s": name_ids.get("solar_system") or [],
            "r": name_ids.get("region") or [],
        }

        for r in hit_ids['r']:
            constellations = providers.esi.client.Universe.get_universe_regions_region_id(
                region_id=r).result()["constellations"]
            for c in constellations:
                if c not in hit_ids["c"]:
                    hit_ids["c"].append(c)

        for c in hit_ids['c']:
            systems = providers.esi.client.Universe.get_universe_constellations_constellation_id(
                constellation_id=c).result()["systems"]
            for s in systems:
                if s not in hit_ids["s"]:
                    hit_ids["s"].append(s)

        sov_structures = providers.esi.client.Sovereignty.get_sovereignty_structures().result()

        hits = []
        names = []
        alliances = []

        for s in sov_structures:
            start = s.get('vulnerable_start_time', False)
            if start:
                if s.get('solar_system_id') in hit_ids["s"] or s.get('alliance_id') in hit_ids["a"]:
                    alliances.append(s.get('alliance_id'))
                    names.append(s.get('solar_system_id'))
                    names.append(s.get('structure_type_id'))
                    hits.append(s)

        if len(names) == 0:
            await ctx.send(":sad: Nothing found for '{}'".format(name_search))
            return True

        names_alli = {}
        for a in set(alliances):
            res = providers.esi.client.Alliance.get_alliances_alliance_id(
                alliance_id=a).result()
            names_alli[a] = res.get("ticker")

        names = providers.esi.client.Universe.post_universe_names(
            ids=list(set(names))).result()

        nms = {}
        for n in names:
            nms[n.get("id")] = n.get("name")

        for hit in hits:
            hit['system_name'] = nms[hit.get('solar_system_id')]
            if hit.get("structure_type_id") == 32226:
                hit['structure'] = "TCU"
            elif hit.get("structure_type_id") == 32458:
                hit['structure'] = "IHUB"
            else:
                hit['structure'] = "¯\\_(ツ)_/¯"

            hit['alliance_name'] = names_alli[hit.get('alliance_id')]

        output = []
        base_str = "**{}** {} (ADM {})[**{}**] Vulnerable{}"
        dt_now = pendulum.now(tz="UTC")
        for h in sorted(hits, key=lambda k: k['vulnerable_start_time']):
            time = ""
            time = " for **{}**".format(pendulum.now(tz="UTC").diff_for_humans(
                h['vulnerable_end_time'], absolute=True
            ))

            if h['vulnerable_start_time']:
                if h['vulnerable_start_time'] > dt_now:
                    time = " in **{}**".format(pendulum.now(tz="UTC").diff_for_humans(
                        h['vulnerable_start_time'], absolute=True
                    ))
            output.append(
                base_str.format(
                    h['system_name'],
                    h['structure'],
                    h['vulnerability_occupancy_level'],
                    h['alliance_name'],
                    time
                )
            )

        n = 15
        chunks = [list(output[i * n:(i + 1) * n])
                  for i in range((len(output) + n - 1) // n)]

        for c in chunks:
            await ctx.send("\n".join(c))
        return True

    @commands.command(pass_context=True)
    @message_in_channels(settings.SOV_DISCORD_BOT_CHANNELS)
    async def lowadm(self, ctx):
        """
        Timers for region/constelation/system/alliance
        """
        await ctx.trigger_typing()

        own_ids = settings.DISCORD_BOT_SOV_STRUCTURE_OWNER_IDS

        include_regions = settings.DISCORD_BOT_ADM_REGIONS
        include_systems = settings.DISCORD_BOT_ADM_SYSTEMS
        include_constel = settings.DISCORD_BOT_ADM_CONSTELLATIONS

        sov_structures = providers.esi.client.Sovereignty.get_sovereignty_structures().result()

        names = {}
        alliances = []
        for s in sov_structures:
            if s.get('alliance_id') in own_ids:
                if s.get('vulnerability_occupancy_level'):
                    if s.get('vulnerability_occupancy_level') < app_settings.get_low_adm():
                        names[s.get('solar_system_id')] = {
                            "system_name": s.get('solar_system_id'),
                            "adm": s.get('vulnerability_occupancy_level')
                        }

        if len(names) == 0:
            await ctx.send(f"All above {app_settings.get_low_adm()}! :ok_hand:")
            return True

        systems = [k for k, v in names.items()]
        constelations = {}
        region_id = {}
        for n in systems:
            system = providers.esi.client.Universe.get_universe_systems_system_id(
                system_id=n).result()
            names[n]["system_name"] = system.get("name")
            names[n]["system_id"] = system.get("system_id")
            names[n]["constellation_id"] = system.get("constellation_id")
            if system.get("constellation_id") not in constelations:
                constelations[system.get("constellation_id")] = {}

        for c, v in constelations.items():
            const = providers.esi.client.Universe.get_universe_constellations_constellation_id(
                constellation_id=c).result()
            region = providers.esi.client.Universe.get_universe_regions_region_id(
                region_id=const.get("region_id")).result()
            v["region_id"] = const.get("region_id")
            v["region_name"] = region.get("name")
            v["constellation_name"] = const.get("name")

        out_array = {}
        for k, v in names.items():
            out_array[k] = {**v, **constelations[v["constellation_id"]]}

        output = {}
        base_str = "**{}** ADM:{}"
        urls = {}
        for k, h in sorted(out_array.items(), key=lambda e: e[1]['adm']):
            show = False
            if h['region_id'] in include_regions:
                show = True
            elif h['constellation_id'] in include_constel:
                show = True
            elif h['system_id'] in include_systems:
                show = True
            if show:
                if h['region_name'] not in output:
                    output[h['region_name']] = []
                output[h['region_name']].append(
                    base_str.format(
                        h['system_name'],
                        h['adm']
                    )
                )
                if h['region_name'].replace(" ", "_") not in urls:
                    urls[h['region_name'].replace(" ", "_")] = []
                urls[h['region_name'].replace(" ", "_")].append(
                    h['system_name'].replace(" ", "_")
                )
        url = "https://evemaps.dotlan.net/map/{}/{}#adm"

        if len(output) > 0:
            for k, v in output.items():
                n = 25
                chunks = [list(v[i * n:(i + 1) * n])
                          for i in range((len(v) + n - 1) // n)]
                for chunk in chunks:
                    await ctx.send("__{}__\n{}".format(k, "\n".join(chunk)))
            await ctx.send(url.format(k.replace(" ", "_"), ",".join(urls[k.replace(" ", "_")])))

        else:
            await ctx.send(f"No Systems with ADM below {app_settings.get_low_adm()}")

        embed = Embed(title="Disclaimer")
        embed.set_thumbnail(
            url="https://avatars3.githubusercontent.com/u/39349660?s=200&v=4"
        )
        embed.colour = Color.red()
        embed.description = "Due to an ESI bug this data might only update once a day at around 2200-2300 Eve Time. **YMMY**\n[For more info please see this issue](https://github.com/esi/esi-issues/issues/1092)"
        await ctx.send(embed=embed)

        return True


def setup(bot):
    bot.add_cog(Sov(bot))




# Cog Stuff
from discord.ext import commands
from discord.embeds import Embed
from discord.colour import Color
from django.conf import settings
# AA Contexts
from aadiscordbot.app_settings import get_site_url
from allianceauth.services.modules.discord.models import DiscordUser

import re

import logging
logger = logging.getLogger(__name__)


class EasterEggs(commands.Cog):
    """
    Stupid commands that don't belong anywhere
    These will not appear in Help menus
    Have limited to no real use
    I was bored
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True, hidden=True)
    async def happybirthday(self, ctx):
        """
        Takes one Discord User as an argument, Wishes this user a happy birthday
        If no user is passed, responds to the context user
        "Useful" to verify the bot is alive and functioning
        """
        await ctx.trigger_typing()

        birthday_user = ctx.message.content[15:]

        if birthday_user == "":
            birthday_user = ctx.message.author.mention
        else:
            pass

        payload = f"Happy Birthday {birthday_user}"
        return await ctx.send(payload)


def setup(bot):
    bot.add_cog(EasterEggs(bot))
